﻿using MathNet.Numerics.Distributions;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;

namespace RandomTemporalGraphGenerator
{
    class Program
    {
        static void Main(string[] args)
        {
            /* Arguments are as follows
             * 0. Number of vertices
             * 1. Probability
             */

            var vertices = int.Parse(args[0]);
            var probability = double.Parse(args[1], CultureInfo.InvariantCulture);

            var seed = (int)((long)((DateTime.Now - new DateTime(1970, 1, 1)).TotalMilliseconds) & 0x0000_0000_ffff_ffff);
            var random = new Random(seed);
            var poisson = new Poisson(probability, random);
            var edgeList = new List<(int u, int v, double t)>();
            int e = 0;

            for (int u = 0; u < vertices; u++)
            {
                for (int v = u + 1; v < vertices; v++)
                {
                    int count = poisson.Sample();
                    for (int i = 0; i < count; i++)
                    {
                        // random vertex order if input interpreted as directed
                        if(random.NextDouble() > 0.5)
                        {
                            edgeList.Add((v, u, 0));
                        }
                        else
                        {
                            edgeList.Add((u, v, 0));
                        }
                    }

                    e += count;
                }
            }

            // assign time labels
            var edges = CollectionsMarshal.AsSpan(edgeList);
            for (int i = 0; i < edges.Length; i++)
            {
                ref var tuple = ref edges[i];
                tuple.t = random.NextDouble();
            }
            edgeList.Sort((a, b) => a.t.CompareTo(b.t));


            var filename = $"random_s{seed}-v{vertices}-p{args[1]}.temporalgraph";
            using var writer = new StreamWriter(filename);
            for (int t = 0; t < edgeList.Count; t++)
            {
                var (u, v, _) = edges[t];
                writer.WriteLine($"{u} {v} {t}");
            }
        }
    }
}
