﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;

namespace TemporalSourceCover.DatasetTransformation
{
    public sealed class BitcoinTransformer : IDatasetTransformer
    {
        public async IAsyncEnumerable<DatasetEdge> Transform(StreamReader reader)
        {
            while (!reader.EndOfStream)
            {
                string line = (await reader.ReadLineAsync())!;
                string[] values = line.Split(',');

                // format: source,target,rating,time
                // time is unix timestamp with decimals
                long u = long.Parse(values[0]);
                long v = long.Parse(values[1]);

                double timestamp = double.Parse(values[3], CultureInfo.InvariantCulture);
                DateTime dateTime = new DateTime(1970, 1, 1) + TimeSpan.FromSeconds(timestamp);

                yield return new DatasetEdge(u, v, dateTime.Ticks);
            }
        }
    }
}
