﻿using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;

namespace TemporalSourceCover.DatasetTransformation
{
    public sealed class MoocTransformer : IDatasetTransformer
    {
        public async IAsyncEnumerable<DatasetEdge> Transform(StreamReader reader)
        {
            // userid and targetid use the same ids while referring to different nodes (as the graph is bipartite)
            var userVertices = new Dictionary<long, long>();
            var targetVertices = new Dictionary<long, long>();

            long nextId = 0;

            bool firstLine = true;
            while (!reader.EndOfStream)
            {
                string line = (await reader.ReadLineAsync())!;

                if (firstLine)
                {
                    firstLine = false;
                    continue;
                }

                yield return TransformLine(line, userVertices, targetVertices, ref nextId);
            }
        }

        private DatasetEdge TransformLine(string line, Dictionary<long, long> userVertices, Dictionary<long, long> targetVertices, ref long nextId)
        {
            // format is "action userid targetid timestamp"
            string[] values = Regex.Split(line, @"\s+").Where(s => s.Length > 0).ToArray();

            long id = nextId;

            long u = userVertices.GetOrAdd(long.Parse(values[1]), () => id++);
            long v = targetVertices.GetOrAdd(long.Parse(values[2]), () => id++);

            nextId = id;

            //timestamp is in double format, but is actually an integer representing seconds
            double time = double.Parse(values[3], CultureInfo.InvariantCulture);

            return new DatasetEdge(u, v, (long)time);
        }
    }
}
