﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;

namespace TemporalSourceCover.DatasetTransformation
{
    public sealed class RedditHyperlinksTransformer : IDatasetTransformer
    {
        public async IAsyncEnumerable<DatasetEdge> Transform(StreamReader reader)
        {
            var subredditIds = new Dictionary<string, long>(StringComparer.OrdinalIgnoreCase);
            long nextSubredditId = 0;

            string dateTimeFormat = "yyyy-MM-dd HH:mm:ss";

            bool firstLine = true;

            while (!reader.EndOfStream)
            {
                string line = (await reader.ReadLineAsync())!;

                if (firstLine)
                {
                    firstLine = false;
                    continue;
                }

                string[] values = line.Split('\t');

                //source target postid timestamp ...
                string source = values[0];
                string target = values[1];
                string timestamp = values[3];

                long u = subredditIds.GetOrAdd(source, () => nextSubredditId++);
                long v = subredditIds.GetOrAdd(target, () => nextSubredditId++);
                long t = DateTime.ParseExact(timestamp, dateTimeFormat, CultureInfo.InvariantCulture).Ticks;

                yield return new DatasetEdge(u, v, t);
            }
        }
    }
}
