﻿using System;
using System.Collections.Generic;
using System.IO;

namespace TemporalSourceCover.DatasetTransformation
{
    public sealed class TransportNetworkTransformer : IDatasetTransformer
    {
        public async IAsyncEnumerable<DatasetEdge> Transform(StreamReader reader)
        {
            var nodes = new Dictionary<string, long>();
            long nodeId = 0;

            bool firstLine = true;
            while (!reader.EndOfStream)
            {
                string line = (await reader.ReadLineAsync())!;
                if (firstLine)
                {
                    firstLine = false;
                    continue;
                }


                // format is node1,node2,layer1,layer2,timestamp,weight
                string[] values = line.Split(',');

                long u = nodes.GetOrAdd(values[0], () => nodeId++);
                long v = nodes.GetOrAdd(values[1], () => nodeId++);

                var datetime = DateTime.ParseExact(values[4], "yyyy-MM-ddTHH:mm:sszzz", null);
                long t = datetime.Ticks;

                yield return new DatasetEdge(u, v, t);
            }
        }
    }
}
