﻿using System.IO;
using System.Text;
using System.Threading.Tasks;

#nullable disable

namespace TemporalSourceCover.Utility
{
    public sealed class ConsoleFileWriter : TextWriter
    {
        private readonly TextWriter _consoleOut;
        private readonly TextWriter _fileOut;

        public override Encoding Encoding => _consoleOut.Encoding;

        public ConsoleFileWriter(TextWriter consoleOut, string file)
        {
            _consoleOut = consoleOut;
            _fileOut = new StreamWriter(file) { AutoFlush = true };
        }

        public override void Flush()
        {
            _consoleOut.Flush();
            _fileOut.Flush();
        }

        protected override void Dispose(bool disposing)
        {
            _consoleOut.Flush();
            _fileOut.Flush();

            _consoleOut.Dispose();
            _fileOut.Dispose();
        }

        public override void Write(char value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(char[] buffer)
        {
            _consoleOut.Write(buffer);
            _fileOut.Write(buffer);

        }

        public override void Write(char[] buffer, int index, int count)
        {
            _consoleOut.Write(buffer, index, count);
            _fileOut.Write(buffer, index, count);

        }

        public override void Write(bool value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(int value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(uint value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(long value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(ulong value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(float value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(double value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(decimal value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(string value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(object value)
        {
            _consoleOut.Write(value);
            _fileOut.Write(value);

        }

        public override void Write(string format, object arg0)
        {
            _consoleOut.Write(format, arg0);
            _fileOut.Write(format, arg0);

        }

        public override void Write(string format, object arg0, object arg1)
        {
            _consoleOut.Write(format, arg0, arg1);
            _fileOut.Write(format, arg0, arg1);

        }

        public override void Write(string format, object arg0, object arg1, object arg2)
        {
            _consoleOut.Write(format, arg0, arg1, arg2);
            _fileOut.Write(format, arg0, arg1, arg2);

        }

        public override void Write(string format, params object[] arg)
        {
            _consoleOut.Write(format, arg);
            _fileOut.Write(format, arg);

        }

        public override void WriteLine()
        {
            _consoleOut.WriteLine();
            _fileOut.WriteLine();

        }

        public override void WriteLine(char value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(char[] buffer)
        {
            _consoleOut.WriteLine(buffer);
            _fileOut.WriteLine(buffer);

        }

        public override void WriteLine(char[] buffer, int index, int count)
        {
            _consoleOut.WriteLine(buffer, index, count);
            _fileOut.WriteLine(buffer, index, count);

        }

        public override void WriteLine(bool value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(int value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(uint value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(long value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(ulong value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(float value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(double value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(decimal value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(string value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(object value)
        {
            _consoleOut.WriteLine(value);
            _fileOut.WriteLine(value);

        }

        public override void WriteLine(string format, object arg0)
        {
            _consoleOut.WriteLine(format, arg0);
            _fileOut.WriteLine(format, arg0);

        }

        public override void WriteLine(string format, object arg0, object arg1)
        {
            _consoleOut.WriteLine(format, arg0, arg1);
            _fileOut.WriteLine(format, arg0, arg1);

        }

        public override void WriteLine(string format, object arg0, object arg1, object arg2)
        {
            _consoleOut.WriteLine(format, arg0, arg1, arg2);
            _fileOut.WriteLine(format, arg0, arg1, arg2);

        }

        public override void WriteLine(string format, params object[] arg)
        {
            _consoleOut.WriteLine(format, arg);
            _fileOut.WriteLine(format, arg);

        }

        public override async Task WriteAsync(char value)
        {
            await _consoleOut.WriteAsync(value);
            await _fileOut.WriteAsync(value);

        }

        public override async Task WriteAsync(string value)
        {
            await _consoleOut.WriteAsync(value);
            await _fileOut.WriteAsync(value);

        }

        public override async Task WriteAsync(char[] buffer, int index, int count)
        {
            await _consoleOut.WriteAsync(buffer, index, count);
            await _fileOut.WriteAsync(buffer, index, count);

        }

        public override async Task WriteLineAsync(char value)
        {
            await _consoleOut.WriteLineAsync(value);
            await _fileOut.WriteLineAsync(value);

        }

        public override async Task WriteLineAsync(string value)
        {
            await _consoleOut.WriteLineAsync(value);
            await _fileOut.WriteLineAsync(value);

        }

        public override async Task WriteLineAsync(char[] buffer, int index, int count)
        {
            await _consoleOut.WriteLineAsync(buffer, index, count);
            await _fileOut.WriteLineAsync(buffer, index, count);

        }

        public override async Task WriteLineAsync()
        {
            await _consoleOut.WriteLineAsync();
            await _fileOut.WriteLineAsync();

        }

        public override async Task FlushAsync()
        {
            await _consoleOut.FlushAsync();
            await _fileOut.FlushAsync();

        }
    }
}
