﻿namespace System.Collections.Generic
{
    public static class DictionaryExtensions
    {
        /// <summary>
        /// Returns the value to the specified key, if present. Otherwise invokes the value factory, adds the pair to the dictionary and returns the value.
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="dictionary"></param>
        /// <param name="key"></param>
        /// <param name="valueIfNotPresent"></param>
        /// <returns></returns>
        public static TValue GetOrAdd<TKey, TValue>(this Dictionary<TKey, TValue> dictionary, TKey key, Func<TValue> valueFactory)
            where TKey : notnull
        {
            if (dictionary.TryGetValue(key, out var value))
                return value;

            value = valueFactory.Invoke();
            dictionary.Add(key, value);
            return value;
        }

        public static void AddOrUpdate<TKey, TValue>(this Dictionary<TKey, TValue> dictionary, TKey key, Func<TValue?, TValue> valueFactory)
            where TKey : notnull
        {
            if (!dictionary.TryGetValue(key, out var existingValue))
            {
                dictionary.Add(key, valueFactory.Invoke(default));
                return;
            }

            dictionary[key] = valueFactory.Invoke(existingValue);
        }
    }
}
