﻿namespace System.Collections.Generic
{
    public static class HashSetExtensions
    {
        /// <summary>
        /// Adds the sets of values to the HashSet
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="set"></param>
        /// <param name="values"></param>
        public static void AddRange<T>(this HashSet<T> set, IEnumerable<T> values)
        {
            foreach (var value in values)
            {
                set.Add(value);
            }
        }

        /// <summary>
        /// Removes all values from the HashSet and returns the values that were actually removed
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="set"></param>
        /// <param name="values"></param>
        /// <returns></returns>
        public static HashSet<T> RemoveRange<T>(this HashSet<T> set, IEnumerable<T> values)
        {
            var removed = new HashSet<T>();

            foreach (var value in values)
            {
                if (set.Remove(value))
                {
                    removed.Add(value);
                }
            }

            return removed;
        }
    }
}
