﻿using System;
using System.Diagnostics;

namespace TemporalSourceCover.Utility
{
    public sealed class Metrics : IDisposable
    {
        private readonly bool _log;
        private readonly Stopwatch _stopwatch;
        public TimeSpan Time => _stopwatch.Elapsed;

        private Metrics(string? title, bool log)
        {
            if (title is not null && log)
            {
                Console.WriteLine(title);
            }

            _log = log;
            _stopwatch = Stopwatch.StartNew();
        }

        public void Dispose()
        {
            _stopwatch.Stop();

            if (_log)
            {
                Console.WriteLine($"{_stopwatch.Elapsed.TotalSeconds:0.00}s");
            }
        }


        public static Metrics BeginMeasurement(string? title, bool log = true)
        {
            return new Metrics(title, log);
        }
    }
}
