﻿using System;

namespace TemporalSourceCover
{
    public struct DataReductionResult
    {
        /// <summary>
        /// Data Reduction Rule
        /// </summary>
        public readonly DataReductionRule Rule;

        /// <summary>
        /// Number of vertices removed from the graph
        /// </summary>
        public long RemovedVertices;

        /// <summary>
        /// Number of edges removed from the graph
        /// </summary>
        public long RemovedEdges;

        /// <summary>
        /// Number of vertices added to the solution
        /// </summary>
        public long SolutionVertices;

        /// <summary>
        /// Running time to apply the data reduction rule
        /// </summary>
        public TimeSpan Time;

        /// <summary>
        /// Indicated whether the rule reduced the data size
        /// </summary>
        public bool WasApplied;

        /// <summary>
        /// Number of vertices that are ignored for a minimal solution
        /// </summary>
        public long IgnoredVertices;

        public DataReductionResult(DataReductionRule rule) : this()
        {
            Rule = rule;
        }
    }
}
