﻿using System.Collections.Generic;
using System.Linq;

namespace TemporalSourceCover
{
    public sealed class TemporalSourceCoverInstance
    {
        public readonly TemporalGraph Graph;
        public int K;

        // Reached vertices are omitted here as they are already accessible through Graph.RedVertices

        public readonly HashSet<TemporalGraphVertex> ForbiddenVertices;

        public IEnumerable<TemporalGraphVertex> PossibleSources => Reachability?.BySource.Keys
            ?? Graph.VerticesByType[VertexType.Regular]
                    .Concat(Graph.VerticesByType[VertexType.Source])
                    .Concat(Graph.VerticesByType[VertexType.Isolated].Where(v => !v.IsRed))
                    .Except(ForbiddenVertices);

        public TemporalGraphReachability? Reachability;

        public TemporalSourceCoverInstance(TemporalGraph graph, int k, HashSet<TemporalGraphVertex> forbiddenVertices)
        {
            Graph = graph;
            K = k;
            ForbiddenVertices = forbiddenVertices;
        }
    }
}
