﻿using System;

namespace TemporalSourceCover
{
    sealed class TemporalSourceCoverTransaction : IDisposable
    {
        private readonly TemporalGraphTransaction _graphTransaction;
        private readonly TemporalGraphReachabilityTransaction? _reachabilityTransaction;

        public TemporalSourceCoverTransaction(TemporalSourceCoverInstance instance)
        {
            _graphTransaction = new TemporalGraphTransaction(instance.Graph);

            if (instance.Reachability is not null)
            {
                _reachabilityTransaction = new TemporalGraphReachabilityTransaction(instance.Reachability);
            }
        }

        public void Dispose()
        {
            _graphTransaction.Dispose();
            _reachabilityTransaction?.Dispose();
        }

        internal void Commit()
        {
            _graphTransaction.Commit();
            _reachabilityTransaction?.Commit();
        }

        internal void Rollback()
        {
            _graphTransaction.Rollback();
            _reachabilityTransaction?.Rollback();
        }
    }
}
